<?php

class Model_Helper_CourseWorkBuilder extends Model_Helper_Builder
{
    const COURSEWORK_DISCIPLINE_ID = 346;  # todo: move to config

    public function create() {
        $this->data += [
            'Lectures'  => 0,
            'Practice'  => 0,
            'Labs'      => 0,
            'Milestone' => 0,
            'IsLocked'  => false,
            'Type'      => 'grading_credit',
        ];

        $required = [
            'Subtype', 'IsLocked', 'Milestone', 'AuthorID', 'GradeID', 'FacultyID',
            'SubjectID', 'Lectures', 'Practice', 'Labs', 'Type', 'SemesterID'
        ];

        if (array_diff($required, array_keys($this->data)))
            throw new InvalidArgumentException('not enough arguments');

        return new Model_CourseWork($this->data, false);
    }

    function & author($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('author id is incorrect');
        $this->data['AuthorID'] = (int) $id;
        return $this;
    }

    function & grade($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('grade id is incorrect');
        $this->data['GradeID'] = (int) $id;
        return $this;
    }

    function & faculty($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('faculty id is incorrect');
        $this->data['FacultyID'] = (int) $id;
        return $this;
    }

    function & subject($id) {
        if ($this->data['Subtype'] === Model_CourseWork::SCIENTIFIC)
            $id = self::COURSEWORK_DISCIPLINE_ID;

        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('subject id is incorrect');

        $this->data['SubjectID'] = (int) $id;
        return $this;
    }

    function & subtype($name) {
        if (!is_string($name))
            throw new InvalidArgumentException('Type is incorrect');

        switch ($name) {
            case Model_CourseWork::SCIENTIFIC:
                $this->data['SubjectID'] = self::COURSEWORK_DISCIPLINE_ID;
                break;

            case Model_CourseWork::DISCIPLINARY:
                break;

            default:
                throw new InvalidArgumentException('Subtype is incorrect');
        }

        $this->data['Subtype'] = $name;
        return $this;
    }

    function & semester($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('semester id is incorrect');
        $this->data['SemesterID'] = (int) $id;
        return $this;
    }
}