<?php defined('SYSPATH') || die('No direct script access.');

/**
 * Class Model_Plan
 *
 * @property-read   $ID             int
 * @property        $ExternalID     string
 * @property        $Year           int
 * @property        $FacultyID      int
 */
class Model_Plan extends Model_Container
{
    protected function getRawData($id) {
        $sql = 'CALL `Plan_GetInfo`(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('Study plan not found');

        return $info->offsetGet(0);
    }

    public static function make() {
        return new Model_Helper_PlanBuilder();
    }

    protected function create() {
        $sql = 'SELECT `Plan_Create`(ExternalID, Year, FacultyID) AS `Identifier`';
        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()->get('Identifier');
    }

    /**
     * @param string $extid
     * @return Model_Plan
     */
    public static function withExternalID($extid) {
        $sql = 'SELECT `Plan_GetIDFromExternalID`(:extid) AS `ID`';
        $id = DB::query(Database::SELECT, $sql)
            ->param(':extid', $extid)
            ->execute()->get('ID');

        return $id != -1 ? Model_Plan::load($id) : null;
    }

    /** @return Model_Discipline[] */
    public function getDisciplines($semesterID = null) {
        $sql = 'CALL Plan_GetDisciplines(:id, :semester)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semester', $semesterID)
            ->execute();

        // todo проерить на работоспособность
        // скорей всего, код, написанный после комментария, не работает и его необходимо заменить на этот:
        //$disciplines = array();
        //foreach ($info as $discipline) {
        //    $disciplines[] = Model_Discipline::load($discipline['ID']);
        //}
        $disciplines = array();
        foreach ($info as $disciplineID) {
            $disciplines[] = Model_Discipline::load($disciplineID);
        }

        return $disciplines;
    }

    /** @return Model_Teacher[] */
    public function getTeachers($loadAll = false, $semesterID = null) {
        $sql = 'CALL Plan_GetTeachers(:id, :semester, :loadAll)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semester', $semesterID)
            ->param(':loadAll', $loadAll)
            ->execute();

        // todo проерить на работоспособность
        $teachers = array();
        foreach ($info as $teacherID) {
            $teachers[] = Model_Teacher::load($teacherID);
        }

        return $teachers;
    }

    public function changeInfo($year, $facultyID) {
        $this->Year = $year;
        $this->FacultyID = $facultyID;
        $this->update();
    }

    public function update() {
        $sql = 'SELECT `Plan_ChangeInfo`(:id, :year, :faculty) AS `ErrorCode`';
        $res = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':year', $this->Year)
            ->param(':faculty', $this->FacultyID)
            ->execute();
        if ($res->get('ErrorCode') < 0) {
            throw new Database_Exception('Plan_ChangeInfo update routine failed!');
        }
    }

    public function bindDiscipline($disciplineID) {
        $sql = 'SELECT `Plan_BindDiscipline`(:id, :discipline) AS `ID`';
        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':discipline', $disciplineID)
            ->execute();
    }
}
