<?php defined('SYSPATH') || die('No direct script access.');

class Model_Students extends Model
{
    /** @return Model_Student[] */
    public static function ofDiscipline(Model_Discipline $discipline) {
        $sql = 'CALL `Discipline_GetStudents`(:id)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':id', $discipline->ID)
            ->execute();

        $list = [];
        foreach ($result as $row)
            $list[] = new Model_Student($row);
        return $list;
    }

    /**
     * @param int $group group id
     * @param int $semesterID
     * @return Database_Result
     */
    public static function ofGroup($group, $semesterID) {
        return Model_Students::searchStudentsByName('', 0, 0, $group, $semesterID);
    }

    public static function getGradeID($gradeNum, $degree) {
        $degree = Database::instance()->escape($degree);
        $sql = 'SELECT `GetGradeID`(:gradeNum, :degree) AS `ID`';
        return DB::query(Database::SELECT, $sql)
            ->param(':grade', $gradeNum)
            ->param(':degree', $degree)
            ->execute()->get('ID');
    }

    public static function search($facultyID, $gradeID, $groupID, $disciplineID, $name) {
        $sql = 'CALL `SearchStudents`(:grade, :group, :faculty, :name, :discipline)';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':name'       => $name,
                ':faculty'    => (int) $facultyID,
                ':grade'      => (int) $gradeID,
                ':group'      => (int) $groupID,
                ':discipline' => (int) $disciplineID,
            ])->execute()->as_array();
    }

    public static function searchStudentsByName($words, $facultyID = null, $gradeID = null, $groupID = null, $semesterID = null) {
        if (!is_array($words))
            $words = [$words];
        $words += ['', '', ''];

        $sql = 'CALL `GetStudents`(:faculty, :grade, :group, :semester, :word1, :word2, :word3)';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':word1'    => $words[0],
                ':word2'    => $words[1],
                ':word3'    => $words[2],
                ':faculty'  => (int) $facultyID,
                ':grade'    => (int) $gradeID,
                ':group'    => (int) $groupID,
                ':semester' => (int) $semesterID,
            ])->execute()->as_array();
    }
}
